@@ -1,14 +1,14 @@
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
-"""Code for calling ClustalW and parsing its output (OBSOLETE).
+"""Code for calling ClustalW and parsing its output (DEPRECATED).
 
 This module has been superseded by the Bio.AlignIO framework for
 alignment parsing, and the ClustalW command line wrapper in
 Bio.Align.Applications for calling the tool. These are both described
 in the current version of the Biopython Tutorial and Cookbook.
-This means Bio.Clustalw is now obsolete and is likely to be deprecated
-and eventually removed in future releases of Biopython.
+This means Bio.Clustalw is now deprecated and likely to be
+removed in future releases of Biopython.
 
 A set of classes to interact with the multiple alignment command
 line program clustalw. 
@@ -29,6 +29,9 @@ classes:
 o ClustalAlignment
 o MultipleAlignCL"""
 
+import warnings
+warnings.warn("Bio.Clustalw is deprecated. Please use the Bio.AlignIO framework for alignment parsing, and the ClustalW command line wrapper in Bio.Align.Applications for calling the tool. These are both described in the current version of the Biopython Tutorial and Cookbook.", DeprecationWarning)
+
 # standard library
 import os
 import sys
@@ -82,24 +85,24 @@ def parse_file(file_name, alphabet = IUPAC.unambiguous_dna, debug_level = 0):
     handle.close()
 
     #Force this generic alignment into a ClustalAlignment... nasty hack
-    if isinstance(alphabet, Alphabet.Gapped) :
+    if isinstance(alphabet, Alphabet.Gapped):
         alpha = alphabet
-    else :
+    else:
         alpha = Alphabet.Gapped(alphabet)
     clustal_alignment = ClustalAlignment(alpha)
     clustal_alignment._records = generic_alignment._records
-    for record in clustal_alignment._records :
+    for record in clustal_alignment._records:
         record.seq.alphabet = alpha
 
-    try :
+    try:
         clustal_alignment._version = generic_alignment._version
-    except AttributeError :
+    except AttributeError:
         #Missing the version, could be a 3rd party tool's output
         pass
 
     try :       
         clustal_alignment._star_info = generic_alignment._star_info
-    except AttributeError :
+    except AttributeError:
         #Missing the consensus, again, this is not always present
         pass
 
@@ -132,6 +135,7 @@ def do_alignment(command_line, alphabet=None):
                                      stdin=subprocess.PIPE,
                                      stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE,
+                                     universal_newlines=True,
                                      shell=(sys.platform!="win32")
                                      )
     #Use .communicate as can get deadlocks with .wait(), see Bug 2804
@@ -502,7 +506,7 @@ def _test():
     """
     import doctest
     import os
-    if os.path.isdir(os.path.join("..","..","Tests")) :
+    if os.path.isdir(os.path.join("..","..","Tests")):
         print "Runing doctests..."
         cur_dir = os.path.abspath(os.curdir)
         os.chdir(os.path.join("..","..","Tests"))
