@@ -1,44 +1,34 @@
-#
-#
-#
-#
-#   CHECK SELF.MAX_SUPPORT
-#
-#
-#
-#
-
-
-
-#
-# Trees.py 
-#
-# Copyright 2005 by Frank Kauff & Cymon J. Cox. All rights reserved.
+# Copyright 2005-2008 by Frank Kauff & Cymon J. Cox. All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license. Please see the LICENSE file that should have been included
 # as part of this package.
 #
-# Tree class handles phylogenetic trees. Provides a set of methods to read and write newick-format tree
-# descriptions, get information about trees (monphyly of taxon sets, congruence between trees, common ancestors,...)
-# and to manipulate trees (reroot trees, split terminal nodes).
-#
-# Bug reports welcome: fkauff@duke.edu
-#
+# Bug reports welcome: fkauff@biologie.uni-kl.de or on Biopython's bugzilla.
+"""Tree class to handle phylogenetic trees.
 
-import sys, random, sets
+Provides a set of methods to read and write newick-format tree descriptions,
+get information about trees (monphyly of taxon sets, congruence between trees,
+common ancestors,...) and to manipulate trees (reroot trees, split terminal
+nodes).
+"""
+
+import sys, random, copy
 import Nodes
 
 PRECISION_BRANCHLENGTH=6
 PRECISION_SUPPORT=6
+NODECOMMENT_START='[&'
+NODECOMMENT_END=']'
 
 class TreeError(Exception): pass
 
 class NodeData:
     """Stores tree-relevant data associated with nodes (e.g. branches or otus)."""
-    def __init__(self,taxon=None,branchlength=0.0,support=None):
+    def __init__(self,taxon=None,branchlength=0.0,support=None,comment=None):
         self.taxon=taxon
         self.branchlength=branchlength
         self.support=support
+        self.comment=comment
 
 class Tree(Nodes.Chain):
     """Represents a tree using a chain of nodes with on predecessor (=ancestor)
@@ -63,22 +53,37 @@ class Tree(Nodes.Chain):
         self.rooted=rooted
         self.name=name
         root=Nodes.Node(data())
-        self.add(root)
-        self.root=root.id
+        self.root = self.add(root)
         if tree:    # use the tree we have
-            self._add_subtree(parent_id=root.id,tree=self._parse(tree)[0])
+            # if Tree is called from outside Nexus parser, we need to get rid of linebreaks, etc
+            tree=tree.strip().replace('\n','').replace('\r','')
+            # there's discrepancy whether newick allows semicolons et the end
+            tree=tree.rstrip(';')
+            subtree_info, base_info = self._parse(tree)
+            root.data = self._add_nodedata(root.data, [[], base_info])
+            self._add_subtree(parent_id=root.id,tree=subtree_info)
         
     def _parse(self,tree):
         """Parses (a,b,c...)[[[xx]:]yy] into subcomponents and travels down recursively."""
-        
+        #Remove any leading/trailing white space - want any string starting
+        #with " (..." should be recognised as a leaf, "(..."
+        tree = tree.strip()
         if tree.count('(')!=tree.count(')'):
-            raise TreeError, 'Parentheses do not match in (sub)tree: '+tree
+            raise TreeError('Parentheses do not match in (sub)tree: '+tree)
         if tree.count('(')==0: # a leaf
-            colon=tree.rfind(':')   
-            if colon>-1:
+            #check if there's a colon, or a special comment, or both  after the taxon name
+            nodecomment=tree.find(NODECOMMENT_START)
+            colon=tree.find(':')
+            if colon==-1 and nodecomment==-1: # none
+                return [tree,[None]]
+            elif colon==-1 and nodecomment>-1: # only special comment
+                return [tree[:nodecomment],self._get_values(tree[nodecomment:])]
+            elif colon>-1 and nodecomment==-1: # only numerical values
+                return [tree[:colon],self._get_values(tree[colon+1:])]
+            elif colon < nodecomment: # taxon name ends at first colon or with special comment
                 return [tree[:colon],self._get_values(tree[colon+1:])]
             else:
-                return [tree,[None]]
+                return [tree[:nodecomment],self._get_values(tree[nodecomment:])]
         else:
             closing=tree.rfind(')')
             val=self._get_values(tree[closing+1:])
@@ -101,48 +106,72 @@ class Tree(Nodes.Chain):
     
     def _add_subtree(self,parent_id=None,tree=None):
         """Adds leaf or tree (in newick format) to a parent_id. (self,parent_id,tree)."""
-        
         if parent_id is None:
             raise TreeError('Need node_id to connect to.')
         for st in tree:
+            nd=self.dataclass()
+            nd = self._add_nodedata(nd, st)
             if type(st[0])==list: # it's a subtree
-                nd=self.dataclass()
-                if len(st[1])>=2: # if there's two values, support comes first. Is that always so?
-                    nd.support=st[1][0]
-                    if st[1][1] is not None:
-                        nd.branchlength=st[1][1]
-                elif len(st[1])==1: # otherwise it could be real branchlengths or support as branchlengths
-                    if not self.__values_are_support: # default
-                        if st[1][0] is not None:
-                            nd.branchlength=st[1][0]
-                    else:
-                        nd.support=st[1][0]
                 sn=Nodes.Node(nd)
                 self.add(sn,parent_id)
                 self._add_subtree(sn.id,st[0])
             else: # it's a leaf
-                nd=self.dataclass()
                 nd.taxon=st[0]
-                if len(st)>1:
-                    if len(st[1])>=2: # if there's two values, support comes first. Is that always so?
-                        nd.support=st[1][0]
-                        if st[1][1] is not None:
-                            nd.branchlength=st[1][1]
-                    elif len(st[1])==1: # otherwise it could be real branchlengths or support as branchlengths
-                        if not self.__values_are_support: # default
-                            if st[1][0] is not None:
-                                nd.branchlength=st[1][0]
-                        else:
-                            nd.support=st[1][0]
                 leaf=Nodes.Node(nd)
                 self.add(leaf,parent_id)
+
+    def _add_nodedata(self, nd, st):
+        """Add data to the node parsed from the comments, taxon and support.
+        """
+        if isinstance(st[1][-1],str) and st[1][-1].startswith(NODECOMMENT_START):
+            nd.comment=st[1].pop(-1)
+        # if the first element is a string, it's the subtree node taxon
+        elif isinstance(st[1][0], str):
+            nd.taxon = st[1][0]
+            st[1] = st[1][1:]
+        if len(st)>1:
+            if len(st[1])>=2: # if there's two values, support comes first. Is that always so?
+                nd.support=st[1][0]
+                if st[1][1] is not None:
+                    nd.branchlength=st[1][1]
+            elif len(st[1])==1: # otherwise it could be real branchlengths or support as branchlengths
+                if not self.__values_are_support: # default
+                    if st[1][0] is not None:
+                        nd.branchlength=st[1][0]
+                else:
+                    nd.support=st[1][0]
+        return nd
     
     def _get_values(self, text):
         """Extracts values (support/branchlength) from xx[:yyy], xx."""
        
         if text=='':
             return None
-        return [float(t) for t in text.split(':') if t.strip()] 
+        nodecomment = None
+        if NODECOMMENT_START in text: # if there's a [&....] comment, cut it out
+            nc_start=text.find(NODECOMMENT_START)
+            nc_end=text.find(NODECOMMENT_END)
+            if nc_end==-1:
+                raise TreeError('Error in tree description: Found %s without matching %s' \
+                                % (NODECOMMENT_START, NODECOMMENT_END))
+            nodecomment=text[nc_start:nc_end+1]
+            text=text[:nc_start]+text[nc_end+1:]
+
+        # pase out supports and branchlengths, with internal node taxa info
+        values = []
+        taxonomy = None
+        for part in [t.strip() for t in text.split(":")]:
+            if part:
+                try:
+                    values.append(float(part))
+                except ValueError:
+                    assert taxonomy is None, "Two string taxonomies?"
+                    taxonomy = part
+        if taxonomy:
+            values.insert(0, taxonomy)
+        if nodecomment:
+            values.append(nodecomment)
+        return values
    
     def _walk(self,node=None):
         """Return all node_ids downwards from a node."""
@@ -188,7 +217,7 @@ class Tree(Nodes.Chain):
         
         node_id = search_taxon(self,taxon)
         """
-        for id,node in self.chain.items():
+        for id,node in self.chain.iteritems():
             if node.data.taxon==taxon:
                 return id
         return None
@@ -210,11 +239,16 @@ class Tree(Nodes.Chain):
         else:
             prev=self.unlink(id)
             self.kill(id)
-            if not prev==self.root and len(self.node(prev).succ)==1:
-                succ=self.node(prev).succ[0]
-                new_bl=self.node(prev).data.branchlength+self.node(succ).data.branchlength
-                self.collapse(prev)
-                self.node(succ).data.branchlength=new_bl
+            if len(self.node(prev).succ)==1:
+                if prev==self.root: # we deleted one branch of a bifurcating root, then we have to move the root upwards
+                    self.root=self.node(self.root).succ[0]    
+                    self.node(self.root).branchlength=0.0
+                    self.kill(prev)
+                else: 
+                    succ=self.node(prev).succ[0]
+                    new_bl=self.node(prev).data.branchlength+self.node(succ).data.branchlength
+                    self.collapse(prev)
+                    self.node(succ).data.branchlength=new_bl
             return prev
         
     def get_taxa(self,node_id=None):
@@ -241,6 +275,56 @@ class Tree(Nodes.Chain):
     def get_terminals(self):
         """Return a list of all terminal nodes."""
         return [i for i in self.all_ids() if self.node(i).succ==[]]
+    
+    def is_terminal(self,node):
+        """Returns True if node is a terminal node."""
+        return self.node(node).succ==[]
+
+    def is_internal(self,node):
+        """Returns True if node is an internal node."""
+        return len(self.node(node).succ)>0
+
+    def is_preterminal(self,node):
+        """Returns True if all successors of a node are terminal ones."""
+        if self.is_terminal(node):
+            return False not in [self.is_terminal(n) for n in self.node(node).succ]
+        else:
+            return False
+    def count_terminals(self,node=None):
+        """Counts the number of terminal nodes that are attached to a node."""
+        if node is None:
+            node=self.root
+        return len([n for n in self._walk(node) if self.is_terminal(n)])
+
+    def collapse_genera(self,space_equals_underscore=True):
+        """Collapses all subtrees which belong to the same genus (i.e share the same first word in their taxon name."""
+
+        while True:
+            for n in self._walk():
+                if self.is_terminal(n):
+                    continue
+                taxa=self.get_taxa(n)
+                genera=[]
+                for t in taxa:
+                    if space_equals_underscore:
+                        t=t.replace(' ','_')
+                    try:
+                        genus=t.split('_',1)[0]
+                    except:
+                        genus='None'
+                    if genus not in genera:
+                        genera.append(genus)
+                if len(genera)==1:
+                    self.node(n).data.taxon=genera[0]+' <collapsed>'
+                    #now we kill all nodes downstream
+                    nodes2kill=[kn for kn in self._walk(node=n)]
+                    for kn in nodes2kill:
+                        self.kill(kn)
+                    self.node(n).succ=[]
+                    break # break out of for loop because node list from _walk will be inconsistent
+            else: # for loop exhausted: no genera to collapse left
+                break # while
+
 
     def sum_branchlength(self,root=None,node=None):
         """Adds up the branchlengths from root (default self.root) to node.
@@ -267,7 +351,15 @@ class Tree(Nodes.Chain):
         if self.node(node).succ==[]:
             return self.node(node).data.taxon
         else:
-            return sets.Set([self.set_subtree(n) for n in self.node(node).succ])
+            try:
+                return frozenset([self.set_subtree(n) for n in self.node(node).succ])
+            except:
+                print node
+                print self.node(node).succ
+                for n in self.node(node).succ:
+                    print n, self.set_subtree(n)
+                print [self.set_subtree(n) for n in self.node(node).succ]
+                raise
             
     def is_identical(self,tree2):
         """Compare tree and tree2 for identity.
@@ -276,37 +368,39 @@ class Tree(Nodes.Chain):
         """
         return self.set_subtree(self.root)==tree2.set_subtree(tree2.root)
 
-    def is_compatible(self,tree2,threshold):
+    def is_compatible(self,tree2,threshold,strict=True):
         """Compares branches with support>threshold for compatibility.
         
         result = is_compatible(self,tree2,threshold)
         """
 
-        if sets.Set(self.get_taxa())!=sets.Set(tree2.get_taxa()):
-            missing2=list(sets.Set(self.get_taxa())-sets.Set(tree2.get_taxa()))
-            missing1=list(sets.Set(tree2.get_taxa())-sets.Set(self.get_taxa()))
-            if missing1 != [None]: 
+        # check if both trees have the same set of taxa. strict=True enforces this.
+        missing2=set(self.get_taxa())-set(tree2.get_taxa())
+        missing1=set(tree2.get_taxa())-set(self.get_taxa())
+        if strict and (missing1 or missing2):
+            if missing1: 
                 print 'Taxon/taxa %s is/are missing in tree %s' % (','.join(missing1) , self.name)
-            if missing2 != [None]:
+            if missing2:
                 print 'Taxon/taxa %s is/are missing in tree %s' % (','.join(missing2) , tree2.name)
-            raise TreeError, 'Can\'t compare trees with different taxon compositions.'
-        t1=[(sets.Set(self.get_taxa(n)),self.node(n).data.support) for n in self.all_ids() if \
+            raise TreeError('Can\'t compare trees with different taxon compositions.')
+        t1=[(set(self.get_taxa(n)),self.node(n).data.support) for n in self.all_ids() if \
             self.node(n).succ and\
             (self.node(n).data and self.node(n).data.support and self.node(n).data.support>=threshold)]
-        t2=[(sets.Set(tree2.get_taxa(n)),tree2.node(n).data.support) for n in tree2.all_ids() if \
+        t2=[(set(tree2.get_taxa(n)),tree2.node(n).data.support) for n in tree2.all_ids() if \
             tree2.node(n).succ and\
             (tree2.node(n).data and tree2.node(n).data.support and tree2.node(n).data.support>=threshold)]
         conflict=[]
         for (st1,sup1) in t1:
             for (st2,sup2) in t2:
-                if not st1.issubset(st2) and not st2.issubset(st1):
-                    intersect,notin1,notin2=st1 & st2, st2-st1, st1-st2
-                    if intersect:
+                if not st1.issubset(st2) and not st2.issubset(st1):                     # don't hiccup on upstream nodes
+                    intersect,notin1,notin2=st1 & st2, st2-st1, st1-st2                 # all three are non-empty sets
+                    # if notin1==missing1 or notin2==missing2  <==> st1.issubset(st2) or st2.issubset(st1) ???
+                    if intersect and not (notin1.issubset(missing1) or notin2.issubset(missing2)):         # omit conflicts due to missing taxa
                         conflict.append((st1,sup1,st2,sup2,intersect,notin1,notin2))
         return conflict
         
     def common_ancestor(self,node1,node2):
-        """Return the common ancestor that connects to nodes.
+        """Return the common ancestor that connects two nodes.
         
         node_id = common_ancestor(self,node1,node2)
         """
@@ -330,22 +424,37 @@ class Tree(Nodes.Chain):
         result = is_monophyletic(self,taxon_list)
         """
         if isinstance(taxon_list,str):
-            taxon_set=sets.Set([taxon_list])
+            taxon_set=set([taxon_list])
         else:
-            taxon_set=sets.Set(taxon_list)
+            taxon_set=set(taxon_list)
         node_id=self.root
         while 1:
-            subclade_taxa=sets.Set(self.get_taxa(node_id))
+            subclade_taxa=set(self.get_taxa(node_id))
             if subclade_taxa==taxon_set:                                        # are we there?
                 return node_id
             else:                                                               # check subnodes
                 for subnode in self.chain[node_id].succ:
-                    if sets.Set(self.get_taxa(subnode)).issuperset(taxon_set):  # taxon_set is downstream
+                    if set(self.get_taxa(subnode)).issuperset(taxon_set):  # taxon_set is downstream
                         node_id=subnode
                         break   # out of for loop
                 else:
                     return -1   # taxon set was not with successors, for loop exhausted
 
+    def is_bifurcating(self,node=None):
+        """Return True if tree downstream of node is strictly bifurcating."""
+        if node is None:
+            node=self.root
+        if node==self.root and len(self.node(node).succ)==3: #root can be trifurcating, because it has no ancestor
+            return self.is_bifurcating(self.node(node).succ[0]) and \
+                    self.is_bifurcating(self.node(node).succ[1]) and \
+                    self.is_bifurcating(self.node(node).succ[2])
+        if len(self.node(node).succ)==2:
+            return self.is_bifurcating(self.node(node).succ[0]) and self.is_bifurcating(self.node(node).succ[1])
+        elif len(self.node(node).succ)==0:
+            return True
+        else:
+            return False
+
     def branchlength2support(self):
         """Move values stored in data.branchlength to data.support, and set branchlength to 0.0
 
@@ -353,7 +462,7 @@ class Tree(Nodes.Chain):
         been read in as branchlength. 
         """
 
-        for n in self.chain.keys():
+        for n in self.chain:
             self.node(n).data.support=self.node(n).data.branchlength
             self.node(n).data.branchlength=0.0
 
@@ -367,6 +476,14 @@ class Tree(Nodes.Chain):
             if self.node(n).data.support:
                 self.node(n).data.support/=float(nrep)
 
+    def has_support(self,node=None):
+        """Returns True if any of the nodes has data.support != None."""
+        for n in self._walk(node):
+            if self.node(n).data.support:
+                return True
+        else:
+            return False
+
     def randomize(self,ntax=None,taxon_list=None,branchlength=1.0,branchlength_sd=None,bifurcate=True):
         """Generates a random tree with ntax taxa and/or taxa from taxlabels.
     
@@ -380,7 +497,7 @@ class Tree(Nodes.Chain):
             taxon_list=['taxon'+str(i+1) for i in range(ntax)]
         elif not ntax and not taxon_list:
             raise TreeError('Either numer of taxa or list of taxa must be specified.')
-        elif ntax<>len(taxon_list):
+        elif ntax != len(taxon_list):
             raise TreeError('Length of taxon list must correspond to ntax.')
         # initiate self with empty root
         self.__init__()
@@ -405,11 +522,12 @@ class Tree(Nodes.Chain):
 
     def display(self):
         """Quick and dirty lists of all nodes."""
-        table=[('#','taxon','prev','succ','brlen','blen (sum)','support')]
-        for i in self.all_ids():
+        table=[('#','taxon','prev','succ','brlen','blen (sum)','support','comment')]
+        #Sort this to be consistent accross CPython, Jython, etc
+        for i in sorted(self.all_ids()):
             n=self.node(i)
             if not n.data:
-                table.append((str(i),'-',str(n.prev),str(n.succ),'-','-','-'))
+                table.append((str(i),'-',str(n.prev),str(n.succ),'-','-','-','-'))
             else:
                 tx=n.data.taxon
                 if not tx:
@@ -423,11 +541,14 @@ class Tree(Nodes.Chain):
                 support=n.data.support
                 if support is None:
                     support='-'
-                table.append((str(i),tx,str(n.prev),str(n.succ),blength,sum_blength,support))
-        print '\n'.join(['%3s %32s %15s %15s %8s %10s %8s' % l for l in table])
+                comment=n.data.comment
+                if comment is None:
+                    comment='-'
+                table.append((str(i),tx,str(n.prev),str(n.succ),blength,sum_blength,support,comment))
+        print '\n'.join(['%3s %32s %15s %15s %8s %10s %8s %20s' % l for l in table])
         print '\nRoot: ',self.root
 
-    def to_string(self,support_as_branchlengths=False,branchlengths_only=False,plain=True):
+    def to_string(self,support_as_branchlengths=False,branchlengths_only=False,plain=True,plain_newick=False,ladderize=None):
         """Return a paup compatible tree line.
        
         to_string(self,support_as_branchlengths=False,branchlengths_only=False,plain=True)
@@ -463,28 +584,48 @@ class Tree(Nodes.Chain):
                         return '%1.2f:0.00000' % (data.support)
                     else:
                         return '0.00:0.00000'
+        def ladderize_nodes(nodes,ladderize=None):
+            """Sorts node numbers according to the number of terminal nodes."""
+            if ladderize in ['left','LEFT','right','RIGHT']:
+                succnode_terminals=[(self.count_terminals(node=n),n) for n in nodes]
+                succnode_terminals.sort()
+                if (ladderize=='right' or ladderize=='RIGHT'):
+                    succnode_terminals.reverse()
+                if succnode_terminals:
+                    succnodes=zip(*succnode_terminals)[1]
+                else:
+                    succnodes=[]
+            else:
+                succnodes=nodes
+            return succnodes
 
-        def newickize(node):
+        def newickize(node,ladderize=None):
             """Convert a node tree to a newick tree recursively."""
 
             if not self.node(node).succ:    #terminal
                 return self.node(node).data.taxon+make_info_string(self.node(node).data,terminal=True)
             else:
-                return '(%s)%s' % (','.join(map(newickize,self.node(node).succ)),make_info_string(self.node(node).data))
-            return subtree
+                succnodes=ladderize_nodes(self.node(node).succ,ladderize=ladderize)
+                subtrees=[newickize(sn,ladderize=ladderize) for sn in succnodes]
+                return '(%s)%s' % (','.join(subtrees),make_info_string(self.node(node).data))
                     
-        treeline='tree '
+        treeline=['tree']
         if self.name:
-            treeline+=self.name
+            treeline.append(self.name)
         else:
-            treeline+='a_tree'
-        treeline+=' = '
-        if self.weight<>1:
-            treeline+='[&W%s] ' % str(round(float(self.weight),3))
+            treeline.append('a_tree')
+        treeline.append('=')
+        if self.weight != 1:
+            treeline.append('[&W%s]' % str(round(float(self.weight),3)))
         if self.rooted:
-            treeline+='[&R] '
-        treeline+='(%s);' % ','.join(map(newickize,self.node(self.root).succ))
-        return treeline 
+            treeline.append('[&R]')
+        succnodes=ladderize_nodes(self.node(self.root).succ)
+        subtrees=[newickize(sn,ladderize=ladderize) for sn in succnodes]
+        treeline.append('(%s)' % ','.join(subtrees))
+        if plain_newick:
+            return treeline[-1]
+        else:
+            return ' '.join(treeline)+';'
         
     def __str__(self):
         """Short version of to_string(), gives plain tree"""
@@ -521,7 +662,8 @@ class Tree(Nodes.Chain):
             elif b1[3]==0 or b2[3]==0:
                 newbranch.append(b1[3]+b2[3]) # one is 0, take the other
             else:
-                raise TreeError, 'Support mismatch in bifurcating root: %f, %f' % (float(b1[3]),float(b2[3]))
+                raise TreeError('Support mismatch in bifurcating root: %f, %f' \
+                                % (float(b1[3]),float(b2[3])))
             self.unrooted.append(newbranch)
 
     def root_with_outgroup(self,outgroup=None):
@@ -533,7 +675,8 @@ class Tree(Nodes.Chain):
                     branch=self.unrooted.pop(i)
                     break 
             else:
-                raise TreeError, 'Unable to connect nodes for rooting: nodes %d and %d are not connected' % (parent,child)
+                raise TreeError('Unable to connect nodes for rooting: nodes %d and %d are not connected' \
+                                % (parent,child))
             self.link(parent,child)
             self.node(child).data.branchlength=branch[2]
             self.node(child).data.support=branch[3]
@@ -565,7 +708,7 @@ class Tree(Nodes.Chain):
                 root_branch=self.unrooted.pop(i)
                 break
         else:
-            raise TreeError, 'Unrooted and rooted Tree do not match'
+            raise TreeError('Unrooted and rooted Tree do not match')
         if outgroup_node==root_branch[1]:
             ingroup_node=root_branch[0]
         else:
@@ -585,15 +728,51 @@ class Tree(Nodes.Chain):
         # if theres still a lonely node in self.chain, then it's the old root, and we delete it
         oldroot=[i for i in self.all_ids() if self.node(i).prev is None and i!=self.root]
         if len(oldroot)>1:
-            raise TreeError, 'Isolated nodes in tree description: %s' % ','.join(oldroot)
+            raise TreeError('Isolated nodes in tree description: %s' \
+                            % ','.join(oldroot))
         elif len(oldroot)==1:
             self.kill(oldroot[0])
         return self.root
         
+    def merge_with_support(self,bstrees=None,constree=None,threshold=0.5,outgroup=None):
+        """Merges clade support (from consensus or list of bootstrap-trees) with phylogeny.
+
+        tree=merge_bootstrap(phylo,bs_tree=<list_of_trees>)
+        or
+        tree=merge_bootstrap(phylo,consree=consensus_tree with clade support)
+        """
+
+        if bstrees and constree:
+            raise TreeError('Specify either list of boostrap trees or consensus tree, not both')
+        if not (bstrees or constree):
+            raise TreeError('Specify either list of boostrap trees or consensus tree.')
+        # no outgroup specified: use the smallest clade of the root
+        if outgroup is None:
+            try:
+                succnodes=self.node(self.root).succ
+                smallest=min([(len(self.get_taxa(n)),n) for n in succnodes]) 
+                outgroup=self.get_taxa(smallest[1])
+            except:
+                raise TreeError("Error determining outgroup.")
+        else: # root with user specified outgroup
+            self.root_with_outgroup(outgroup)
+
+        if bstrees: # calculate consensus 
+            constree=consensus(bstrees,threshold=threshold,outgroup=outgroup)
+        else:
+            if not constree.has_support():
+                constree.branchlength2support()
+            constree.root_with_outgroup(outgroup)
+        # now we travel all nodes, and add support from consensus, if the clade is present in both
+        for pnode in self._walk():
+            cnode=constree.is_monophyletic(self.get_taxa(pnode))
+            if cnode>-1:
+                self.node(pnode).data.support=constree.node(cnode).data.support
+
          
 def consensus(trees, threshold=0.5,outgroup=None):
     """Compute a majority rule consensus tree of all clades with relative frequency>=threshold from a list of trees."""
-    
+
     total=len(trees)
     if total==0:
         return None
@@ -602,15 +781,16 @@ def consensus(trees, threshold=0.5,outgroup=None):
     max_support=trees[0].max_support
     clades={}
     #countclades={}
-    alltaxa=sets.Set(trees[0].get_taxa())
+    alltaxa=set(trees[0].get_taxa())
     # calculate calde frequencies
     c=0
     for t in trees:
         c+=1
-        #if c%50==0:
+        #print c
+        #if c%1==0:
         #    print c
-        if alltaxa!=sets.Set(t.get_taxa()):
-            raise TreeError, 'Trees for consensus must contain the same taxa'
+        if alltaxa!=set(t.get_taxa()):
+            raise TreeError('Trees for consensus must contain the same taxa')
         t.root_with_outgroup(outgroup=outgroup)
         for st_node in t._walk(t.root):
             subclade_taxa=t.get_taxa(st_node)
@@ -625,16 +805,16 @@ def consensus(trees, threshold=0.5,outgroup=None):
             #else:
             #    countclades[subclade_taxa]=t.weight
     # weed out clades below threshold
-    for (c,p) in clades.items():
+    for c, p in clades.iteritems():
         if p<threshold:
             del clades[c]
     # create a tree with a root node
     consensus=Tree(name='consensus_%2.1f' % float(threshold),data=dataclass)
     # each clade needs a node in the new tree, add them as isolated nodes
-    for (c,s) in clades.items():
+    for c, s in clades.iteritems():
         node=Nodes.Node(data=dataclass())
         node.data.support=s
-        node.data.taxon=sets.Set(eval(c))
+        node.data.taxon=set(eval(c))
         consensus.add(node)
     # set root node data
     consensus.node(consensus.root).data.support=None
@@ -665,6 +845,3 @@ def consensus(trees, threshold=0.5,outgroup=None):
     consensus.node(consensus_ids[-1]).data.taxon=None 
     return consensus
 
-    
-                
-        
