@@ -4,7 +4,7 @@
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
-"""Module for working with Prosite files from ExPASy (OBSOLETE).
+"""Module for working with Prosite files from ExPASy (DEPRECATED).
 
 Most of the functionality in this module has moved to Bio.ExPASy.Prosite;
 please see
@@ -55,6 +55,15 @@ _Scanner              Scans Prosite-formatted data.
 _RecordConsumer       Consumes Prosite data to a Record object.
 
 """
+
+import warnings
+warnings.warn("Bio.Prosite is deprecated, and will be removed in a"\
+              " future release of Biopython. Most of the functionality "
+              " is now provided by Bio.ExPASy.Prosite.  If you want to "
+              " continue to use Bio.Prosite, please get in contact "
+              " via the mailing lists to avoid its permanent removal from"\
+              " Biopython.", DeprecationWarning)
+
 from types import *
 import re
 import sgmllib
@@ -62,7 +71,6 @@ from Bio import File
 from Bio import Index
 from Bio.ParserSupport import *
 
-
 # There is probably a cleaner way to write the read/parse functions
 # if we don't use the "parser = RecordParser(); parser.parse(handle)"
 # approach. Leaving that for the next revision of Bio.Prosite.
@@ -539,7 +547,7 @@ class _RecordConsumer(AbstractConsumer):
                 # CC   Automatic scaling using reversed database
                 # Throw it away.  (Should I keep it?)
                 continue
-            if col.count("=") == 0 :
+            if col.count("=") == 0:
                 #Missing qualifier!  Can we recover gracefully?
                 #For example, from Bug 2403, in PS50293 have:
                 #CC /AUTHOR=K_Hofmann; N_Hulo
@@ -722,7 +730,7 @@ def index_file(filename, indexname, rec2key=None):
     end = 0L
     for record in records:
         start = end
-        end = long(handle.tell())
+        end = handle.tell()
         length = end - start
         
         if rec2key is not None:
