@@ -1,3 +1,11 @@
+# Copyright 2000-2003 Jeff Chang.
+# Copyright 2001-2008 Brad Chapman.
+# Copyright 2005-2010 by Peter Cock.
+# Copyright 2006-2009 Michiel de Hoon.
+# All rights reserved.
+# This code is part of the Biopython distribution and governed by its
+# license.  Please see the LICENSE file that should have been included
+# as part of this package.
 """Represent a Sequence Feature holding info about a part of a sequence.
 
 This is heavily modeled after the Biocorba SeqFeature objects, and
@@ -33,16 +41,19 @@ o FeatureLocation - Specify the start and end location of a feature.
 
 o ExactPosition - Specify the position as being exact.
 o WithinPosition - Specify a position occuring within some range.
-o BetweenPosition - Specify a position occuring between a range.
+o BetweenPosition - Specify a position occuring between a range (OBSOLETE?).
 o BeforePosition - Specify the position as being found before some base.
 o AfterPosition - Specify the position as being found after some base.
+o OneOfPosition - Specify a position where the location can be multiple positions.
 """
 
+from Bio.Seq import MutableSeq, reverse_complement
+
 class SeqFeature(object):
     """Represent a Sequence Feature on an object.
 
     Attributes:
-    o location - the location of the feature on the sequence
+    o location - the location of the feature on the sequence (FeatureLocation)
     o type - the specified type of the feature (ie. CDS, exon, repeat...)
     o location_operator - a string specifying how this SeqFeature may
     be related to others. For example, in the example GenBank feature
@@ -64,16 +75,49 @@ class SeqFeature(object):
 
     CDS    join(1..10,30..40,50..60)
 
-    The the top level feature would be a CDS from 1 to 60, and the sub
-    features would be of 'CDS_join' type and would be from 1 to 10, 30 to
-    40 and 50 to 60, respectively.
+    Then the top level feature would be of type 'CDS' from 1 to 60 (actually 0
+    to 60 in Python counting) with location_operator='join', and the three sub-
+    features would also be of type 'CDS', and would be from 1 to 10, 30 to
+    40 and 50 to 60, respectively (although actually using Python counting).
+
+    To get the nucleotide sequence for this CDS, you would need to take the
+    parent sequence and do seq[0:10]+seq[29:40]+seq[49:60] (Python counting).
+    Things are more complicated with strands and fuzzy positions. To save you
+    dealing with all these special cases, the SeqFeature provides an extract
+    method to do this for you.
     """
     def __init__(self, location = None, type = '', location_operator = '',
                  strand = None, id = "<unknown id>", 
                  qualifiers = None, sub_features = None,
                  ref = None, ref_db = None):
         """Initialize a SeqFeature on a Sequence.
+
+        location can either be a FeatureLocation (with strand argument also
+        given if required), or None.
+
+        e.g. With no strand, on the forward strand, and on the reverse strand:
+
+        >>> from Bio.SeqFeature import SeqFeature, FeatureLocation
+        >>> f1 = SeqFeature(FeatureLocation(5,10), type="domain")
+        >>> f2 = SeqFeature(FeatureLocation(7,110), strand=1, type="CDS")
+        >>> f3 = SeqFeature(FeatureLocation(9,108), strand=-1, type="CDS")
+
+        An invalid strand will trigger an exception:
+
+        >>> f4 = SeqFeature(FeatureLocation(50,60), strand=2)
+        Traceback (most recent call last):
+           ...
+        ValueError: Strand should be +1, -1, 0 or None, not 2
+
+        For exact start/end positions, an integer can be used (as shown above)
+        as shorthand for the ExactPosition object. For non-exact locations, the
+        FeatureLocation must be specified via the appropriate position objects.
         """
+        if strand not in [-1, 0, 1, None] :
+            raise ValueError("Strand should be +1, -1, 0 or None, not %s" \
+                             % repr(strand))
+        if location is not None and not isinstance(location, FeatureLocation):
+            raise TypeError("FeatureLocation (or None) required for the location")
         self.location = location
 
         self.type = type
@@ -92,17 +136,17 @@ class SeqFeature(object):
     def __repr__(self):
         """A string representation of the record for debugging."""
         answer = "%s(%s" % (self.__class__.__name__, repr(self.location))
-        if self.type :
+        if self.type:
             answer += ", type=%s" % repr(self.type)
-        if self.location_operator :
+        if self.location_operator:
             answer += ", location_operator=%s" % repr(self.location_operator)
-        if self.strand :
+        if self.strand:
             answer += ", strand=%s" % repr(self.strand)
-        if self.id and self.id != "<unknown id>" :
+        if self.id and self.id != "<unknown id>":
             answer += ", id=%s" % repr(self.id)
-        if self.ref :
+        if self.ref:
             answer += ", ref=%s" % repr(self.ref)
-        if self.ref_db :
+        if self.ref_db:
             answer += ", ref_db=%s" % repr(self.ref_db)
         answer += ")"
         return answer
@@ -112,38 +156,91 @@ class SeqFeature(object):
         """
         out = "type: %s\n" % self.type
         out += "location: %s\n" % self.location
-        out += "ref: %s:%s\n" % (self.ref, self.ref_db)
+        if self.id and self.id != "<unknown id>":
+            out += "id: %s\n" % self.id
+        if self.ref or self.ref_db:
+            out += "ref: %s:%s\n" % (self.ref, self.ref_db)
         out += "strand: %s\n" % self.strand
         out += "qualifiers: \n"
-        qualifier_keys = self.qualifiers.keys()
-        qualifier_keys.sort()
-        for qual_key in qualifier_keys:
+        for qual_key in sorted(self.qualifiers):
             out += "    Key: %s, Value: %s\n" % (qual_key,
                                                self.qualifiers[qual_key])
         if len(self.sub_features) != 0:
             out += "Sub-Features\n"
             for sub_feature in self.sub_features:
                 out +="%s\n" % sub_feature
-
         return out
 
-    def _shift(self, offset) :
+    def _shift(self, offset):
         """Returns a copy of the feature with its location shifted (PRIVATE).
 
         The annotation qaulifiers are copied."""
-        answer = SeqFeature(location = self.location._shift(offset),
-                   type = self.type,
-                   location_operator = self.location_operator,
-                   strand = self.strand,
-                   id = self.id,
-                   #qualifiers = dict(self.qualifiers.iteritems()),
-                   #sub_features = [f._shift(offset) for f in self.sub_features],
-                   ref = self.ref,
-                   ref_db = self.ref_db)
-        #TODO - Sort out the use of sub_feature and qualifiers in __init___
-        answer.sub_features = [f._shift(offset) for f in self.sub_features]
-        answer.qualifiers = dict(self.qualifiers.iteritems())
-        return answer
+        return SeqFeature(location = self.location._shift(offset),
+                type = self.type,
+                location_operator = self.location_operator,
+                strand = self.strand,
+                id = self.id,
+                qualifiers = dict(self.qualifiers.iteritems()),
+                sub_features = [f._shift(offset) for f in self.sub_features],
+                ref = self.ref,
+                ref_db = self.ref_db)
+
+    def extract(self, parent_sequence):
+        """Extract feature sequence from the supplied parent sequence.
+
+        The parent_sequence can be a Seq like object or a string, and will
+        generally return an object of the same type. The exception to this is
+        a MutableSeq as the parent sequence will return a Seq object.
+
+        This should cope with complex locations including complements, joins
+        and fuzzy positions. Even mixed strand features should work! This
+        also covers features on protein sequences (e.g. domains), although
+        here reverse strand features are not permitted.
+
+        >>> from Bio.Seq import Seq
+        >>> from Bio.Alphabet import generic_protein
+        >>> from Bio.SeqFeature import SeqFeature, FeatureLocation
+        >>> seq = Seq("MKQHKAMIVALIVICITAVVAAL", generic_protein)
+        >>> f = SeqFeature(FeatureLocation(8,15), type="domain")
+        >>> f.extract(seq)
+        Seq('VALIVIC', ProteinAlphabet())
+
+        Note - currently only sub-features of type "join" are supported.
+        """
+        if isinstance(parent_sequence, MutableSeq):
+            #This avoids complications with reverse complements
+            #(the MutableSeq reverse complement acts in situ)
+            parent_sequence = parent_sequence.toseq()
+        if self.sub_features:
+            if self.location_operator!="join":
+                raise ValueError(self.location_operator)
+            if self.strand == -1:
+                #This is a special case given how the GenBank parser works.
+                #Must avoid doing the reverse complement twice.
+                parts = []
+                for f_sub in self.sub_features:
+                    assert f_sub.strand==-1
+                    parts.append(parent_sequence[f_sub.location.nofuzzy_start:\
+                                                 f_sub.location.nofuzzy_end])
+            else:
+                #This copes with mixed strand features:
+                parts = [f_sub.extract(parent_sequence) \
+                         for f_sub in self.sub_features]
+            #We use addition rather than a join to avoid alphabet issues:
+            f_seq = parts[0]
+            for part in parts[1:] : f_seq += part
+        else:
+            f_seq = parent_sequence[self.location.nofuzzy_start:\
+                                    self.location.nofuzzy_end]
+        if self.strand == -1:
+            #TODO - MutableSeq?
+            try:
+                f_seq = f_seq.reverse_complement()
+            except AttributeError:
+                assert isinstance(f_seq, str)
+                f_seq = reverse_complement(f_seq)
+        return f_seq
+        
 
 # --- References
 
@@ -221,6 +318,23 @@ class FeatureLocation(object):
         position. In the case of integers, the values are assumed to be
         exact and are converted in ExactPosition arguments. This is meant
         to make it easy to deal with non-fuzzy ends.
+
+        i.e. Short form:
+        
+        >>> from Bio.SeqFeature import FeatureLocation
+        >>> loc = FeatureLocation(5,10)
+        
+        Explicit form:
+
+        >>> from Bio.SeqFeature import FeatureLocation, ExactPosition
+        >>> loc = FeatureLocation(ExactPosition(5),ExactPosition(10))
+
+        Other fuzzy positions are used similarly,
+
+        >>> from Bio.SeqFeature import FeatureLocation
+        >>> from Bio.SeqFeature import BeforePosition, AfterPosition
+        >>> loc2 = FeatureLocation(BeforePosition(5),AfterPosition(10))
+
         """
         if isinstance(start, AbstractPosition):
             self._start = start
@@ -246,7 +360,7 @@ class FeatureLocation(object):
         return "%s(%s,%s)" \
                % (self.__class__.__name__, repr(self.start), repr(self.end))
 
-    def _shift(self, offset) :
+    def _shift(self, offset):
         """Returns a copy of the location shifted by the offset (PRIVATE)."""
         return FeatureLocation(start = self._start._shift(offset),
                                end = self._end._shift(offset))
@@ -257,15 +371,8 @@ class FeatureLocation(object):
     end = property(fget= lambda self : self._end,
                    doc="End location (possibly a fuzzy position, read only).")
 
-    def _get_nofuzzy_start(self) :
-        #TODO - Do we still use the BetweenPosition class?
-        if ((self._start == self._end) and isinstance(self._start,
-             BetweenPosition)):
-            return self._start.position
-        else:
-            return min(self._start.position,
-                       self._start.position + self._start.extension)
-    nofuzzy_start = property(fget=_get_nofuzzy_start,
+    nofuzzy_start = property(
+        fget=lambda self: self._start.position,
         doc="""Start position (integer, approximated if fuzzy, read only).
 
         To get non-fuzzy attributes (ie. the position only) ask for
@@ -274,15 +381,8 @@ class FeatureLocation(object):
         (10.20)..(30.40) should return 10 for start, and 40 for end.
         """)
 
-    def _get_nofuzzy_end(self) :
-        #TODO - Do we still use the BetweenPosition class?
-        if ((self._start == self._end) and isinstance(self._start,
-             BetweenPosition)):
-            return self._end.position
-        else:
-            return max(self._end.position,
-                       self._end.position + self._end.extension)
-    nofuzzy_end = property(fget=_get_nofuzzy_end,
+    nofuzzy_end = property(
+        fget=lambda self: self._end.position + self._end.extension,
         doc="""End position (integer, approximated if fuzzy, read only).
 
         To get non-fuzzy attributes (ie. the position only) ask for
@@ -296,15 +396,21 @@ class AbstractPosition(object):
     """
     def __init__(self, position, extension):
         self.position = position
+        assert extension >= 0, extension
         self.extension = extension
 
-    def __repr__(self) :
+    def __repr__(self):
         """String representation of the location for debugging."""
         return "%s(%s,%s)" % (self.__class__.__name__, \
                               repr(self.position), repr(self.extension))
 
-    def __cmp__(self, other):
-        """A simple comparison function for positions.
+    def __hash__(self):
+        """Simple position based hash."""
+        #Note __hash__ must be implemented on Python 3.x if overriding __eq__
+        return hash(self.position)
+
+    def __eq__(self, other):
+        """A simple equality for positions.
 
         This is very simple-minded and just compares the position attribute
         of the features; extensions are not considered at all. This could
@@ -312,10 +418,64 @@ class AbstractPosition(object):
         """
         assert isinstance(other, AbstractPosition), \
           "We can only do comparisons between Biopython Position objects."
+        return self.position == other.position
 
-        return cmp(self.position, other.position)
+    def __ne__(self, other):
+        """A simple non-equality for positions.
 
-    def _shift(self, offset) :
+        This is very simple-minded and just compares the position attribute
+        of the features; extensions are not considered at all. This could
+        potentially be expanded to try to take advantage of extensions.
+        """
+        assert isinstance(other, AbstractPosition), \
+          "We can only do comparisons between Biopython Position objects."
+        return self.position != other.position
+
+    def __le__(self, other):
+        """A simple less than or equal for positions.
+
+        This is very simple-minded and just compares the position attribute
+        of the features; extensions are not considered at all. This could
+        potentially be expanded to try to take advantage of extensions.
+        """
+        assert isinstance(other, AbstractPosition), \
+          "We can only do comparisons between Biopython Position objects."
+        return self.position <= other.position
+
+    def __lt__(self, other):
+        """A simple less than or equal for positions.
+
+        This is very simple-minded and just compares the position attribute
+        of the features; extensions are not considered at all. This could
+        potentially be expanded to try to take advantage of extensions.
+        """
+        assert isinstance(other, AbstractPosition), \
+          "We can only do comparisons between Biopython Position objects."
+        return self.position < other.position
+
+    def __ge__(self, other):
+        """A simple less than or equal for positions.
+
+        This is very simple-minded and just compares the position attribute
+        of the features; extensions are not considered at all. This could
+        potentially be expanded to try to take advantage of extensions.
+        """
+        assert isinstance(other, AbstractPosition), \
+          "We can only do comparisons between Biopython Position objects."
+        return self.position >= other.position
+
+    def __gt__(self, other):
+        """A simple less than or equal for positions.
+
+        This is very simple-minded and just compares the position attribute
+        of the features; extensions are not considered at all. This could
+        potentially be expanded to try to take advantage of extensions.
+        """
+        assert isinstance(other, AbstractPosition), \
+          "We can only do comparisons between Biopython Position objects."
+        return self.position > other.position
+
+    def _shift(self, offset):
         #We want this to maintain the subclass when called from a subclass
         return self.__class__(self.position + offset, self.extension)
             
@@ -335,7 +495,7 @@ class ExactPosition(AbstractPosition):
                                  % extension)
         AbstractPosition.__init__(self, position, 0)
 
-    def __repr__(self) :
+    def __repr__(self):
         """String representation of the ExactPosition location for debugging."""
         assert self.extension == 0
         return "%s(%s)" % (self.__class__.__name__, repr(self.position))
@@ -362,7 +522,7 @@ class WithinPosition(AbstractPosition):
         return "(%s.%s)" % (self.position, self.position + self.extension)
 
 class BetweenPosition(AbstractPosition):
-    """Specify the position of a boundary between two coordinates.
+    """Specify the position of a boundary between two coordinates (OBSOLETE?).
 
     Arguments:
     o position - The start position of the boundary.
@@ -397,7 +557,7 @@ class BeforePosition(AbstractPosition):
                                  % extension)
         AbstractPosition.__init__(self, position, 0)
 
-    def __repr__(self) :
+    def __repr__(self):
         """A string representation of the location for debugging."""
         assert self.extension == 0
         return "%s(%s)" % (self.__class__.__name__, repr(self.position))
@@ -423,7 +583,7 @@ class AfterPosition(AbstractPosition):
                                  % extension)
         AbstractPosition.__init__(self, position, 0)
 
-    def __repr__(self) :
+    def __repr__(self):
         """A string representation of the location for debugging."""
         assert self.extension == 0
         return "%s(%s)" % (self.__class__.__name__, repr(self.position))
@@ -463,7 +623,7 @@ class OneOfPosition(AbstractPosition):
         # initialize with our definition of position and extension
         AbstractPosition.__init__(self, smallest, largest - smallest)
 
-    def __repr__(self) :
+    def __repr__(self):
         """String representation of the OneOfPosition location for debugging."""
         return "%s(%s)" % (self.__class__.__name__, \
                            repr(self.position_choices))
@@ -476,6 +636,10 @@ class OneOfPosition(AbstractPosition):
         out = out[:-1] + ")"
         return out
 
+    def _shift(self, offset):
+        return self.__class__([position_choice._shift(offset) \
+                               for position_choice in self.position_choices])
+
 class PositionGap(object):
     """Simple class to hold information about a gap between positions.
     """
@@ -484,10 +648,20 @@ class PositionGap(object):
         """
         self.gap_size = gap_size
 
-    def __repr__(self) :
+    def __repr__(self):
         """A string representation of the position gap for debugging."""
         return "%s(%s)" % (self.__class__.__name__, repr(self.gap_size))
     
     def __str__(self):
         out = "gap(%s)" % self.gap_size
         return out
+
+def _test():
+    """Run the Bio.SeqFeature module's doctests."""
+    print "Runing doctests..."
+    import doctest
+    doctest.testmod()
+    print "Done"
+
+if __name__ == "__main__":
+    _test()
